#pragma once

#define		LEAF_I_SIZE		16				/* a x a tiles */
#define		LEAF_V_SIZE		(LEAF_I_SIZE+1)	/* (a+1)x(a+1) vertices */

#define		_S_TERRAIN			"Terrain"
#define		_S_METHOD			"Method"
#define		_S_HEIGHTMAP		"HeightMap"
#define		_S_ITERCOUNT		"IterCount"
#define		_S_MAXHEIGHT		"Maxheight"
#define		_S_DISPLACEMENT		"Displacement"

#define		TF_DRAWQUADSMAP		0x0001
#define		TF_USEQUADS			0x0002

enum TERRAIN_METHOD
{
	TM_HEIGHTMAP = 0,
	TM_CIRCLE
};

struct	TEXTURE_PIXEL
{
	BYTE	b;
	BYTE	g;
	BYTE	r;
	BYTE	a;
};

struct BoundingPoint {

	float	x, y, z;
};

typedef enum NODETYPE {

	NODE_TYPE,
	LEAF_TYPE,
};

#define CULL_TOLERANCE	0.0f 

//
// Quad tree node
struct	QTNode {
	//
	// Type of node - NODE, LEAF
	NODETYPE	ntType;	
	//
	// Bounding points
	BoundingPoint	arBounds[4];
	//
	// Each node has four branches/children/kids
	// LEAF has not any kids
	QTNode		*pBranches[4];
	//
	// leaf's tiles, each LEAF has 25 vertices (2 triangles per tile -> 32 triangles per LEAF)
	// NODE has not any vertices	
	IVertexBuffer *pVB;
	BOOL		bVis;	
};

class CTerrain
{	
	DWORD			m_dwFlags;
	// Common IB, heightmap and material
	IIndexBuffer	*m_pTerrainIB;
	ITexture		*m_pTerrainSurface;
	ITexture		*m_pHeightMap;
	// Terrain dimensions
	int				m_iTerrainTilesX;
	int				m_iTerrainTilesY;
	int				m_iTerrainVerticesX;
	int				m_iTerrainVerticesY;
	// Current terrain method
	TERRAIN_METHOD	m_iTerrainMethod;
	I3DFont			*m_pDebugFont;

	std::vector <QTNode*>	m_arLeaves;
	std::vector <QTNode*>	m_arVisQuads;
	//
	// Root of the quad tree
	QTNode			*m_pRoot;
	//
	// Vertices...
	VERTEX			**m_arTerrain;
	//
	// Common inicialization
	HRESULT			InternalInit();
	int				CreateNode(QTNode *pNode);
	int				FillLeaves();
	int				DestroyNode(QTNode *pNode);
	int				CircleAlgorithm();
	int				ComputeNormals();
	int				CreateQuadTree();
	// Fills queue of the visible quads
	HRESULT			CullTerrain(QTNode *pNode, CLIPVOLUME& cv);

public:
	HRESULT		GenerateTerrain(int x, int y, TERRAIN_METHOD tmMethod = TM_CIRCLE);
	HRESULT		GenerateTerrainFromFile(LPCSTR szFile);
	HRESULT		DestroyTerrain();
	int			GetTitlesX() {return m_iTerrainTilesX;}
	int			GetTitlesY() {return m_iTerrainTilesY;}

	void		SetFlag(DWORD dwFlag) {m_dwFlags |= dwFlag;}
	void		UnsetFlag(DWORD dwFlag) {m_dwFlags &= ~dwFlag;}
	void		InvertFlag(DWORD dwFlag) { if(dwFlag & m_dwFlags) { UnsetFlag(dwFlag); } else { SetFlag(dwFlag);}}
	void		ResetFlags() {m_dwFlags = 0;}

	HRESULT		Render();
	HRESULT		Restore();

	CTerrain(void);
	~CTerrain(void);
};

