#ifndef OBOUSEZNAM_H
  #define OBOUSEZNAM_H

  template<class T> class CObouPrvek {
		private:
			T m_Data;
			CObouPrvek<T> *m_lpDalsi;
			CObouPrvek<T> *m_lpPredchozi;
		public:
			CObouPrvek() : m_lpDalsi(NULL), m_lpPredchozi(NULL) { }
			CObouPrvek(const T& _Data) : m_Data(_Data), m_lpDalsi(NULL), m_lpPredchozi(NULL) { }

			T VratData() const { return m_Data; }
			CObouPrvek<T>* VratDalsi() const { return m_lpDalsi; }
			CObouPrvek<T>* VratPredchozi() const { return m_lpPredchozi; }
			void NastavDalsi(CObouPrvek<T> *_lpDalsi) { m_lpDalsi = _lpDalsi; }
			void NastavPredchozi(CObouPrvek<T> *_lpPredchozi) { m_lpPredchozi = _lpPredchozi; }
			void NastavData(const T& _Data) { m_Data = _Data; }
  };

  template<class T> class CObouSeznam {
		private:
			CObouPrvek<T> *m_lpPrvni, *m_lpPosledni;
		public:
			CObouSeznam() : m_lpPrvni(NULL), m_lpPosledni(NULL) { }
			~CObouSeznam() { if(!JePrazdny()) { SmazSeznam(); } }

			bool VytvorPrazdny();

			bool SmazSeznam();

			bool JePrazdny();

			bool VlozNaKonec(const T& _Data);	// vklada na konec seznamu
			bool VlozNaZacatek(const T& _Data);	// vklada na zacatek seznamu

			CObouPrvek<T>* Najdi(const T& _Data);	// vrati ukazatel na nalezeny prvek nebo NULL
			CObouPrvek<T>* NajdiBezZarazky(const T& _Data);
			CObouPrvek<T>* Najdi(const T& _Data, unsigned int _k);

			bool VlozZa(CObouPrvek<T>* _lpPrvek, const T& _Data);
			bool VlozPred(CObouPrvek<T>* _lpPrvek, const T& _Data);
			
			bool SmazPrvek(CObouPrvek<T>* _lpPrvek);
			bool SmazZa(CObouPrvek<T>* _lpPrvek);
			bool SmazPred(CObouPrvek<T>* _lpPrvek);
			
			CObouPrvek<T>* NajdiNejmensi(CObouPrvek<T>* _lpPrvek);
			void SetridVyberem();
			
			void VypisOdZacatku();
			void VypisOdKonce();
  };

  template<class T> bool CObouSeznam<T>::JePrazdny()
  { 
	  bool bNavrat = true;

	  if(m_lpPosledni) { bNavrat = (m_lpPosledni->VratPredchozi() == m_lpPrvni); }

	  return bNavrat;
  }

  template<class T> bool CObouSeznam<T>::VytvorPrazdny()
  {
	  bool bNavrat = false;

	  if(!JePrazdny())
	  {
		  SmazSeznam();	// smazeme puvodni seznam
	  }

	  // nova inicializace promennych
	  m_lpPrvni = new CObouPrvek<T>;

	  if(m_lpPrvni)
	  {
		  // muzeme pokracovat
		  m_lpPosledni = new CObouPrvek<T>;
		  
		  if(m_lpPosledni)
		  {
			  m_lpPrvni->NastavDalsi(m_lpPosledni);	// NULL do predchoziho prvku da konstruktor
			  m_lpPosledni->NastavPredchozi(m_lpPrvni);	// NULL do nasledujiciho prvku da konstruktor

			  bNavrat = true;
		  }
		  else
		  {
			  delete m_lpPrvni;	// pokud se nevytvoril posledni, smazeme prvni
			  m_lpPrvni = NULL;
		  }
	  }

	  return bNavrat;
  }

  template<class T> bool CObouSeznam<T>::VlozNaZacatek(const T& _Data)
  {
	  bool bNavrat = false;

	  if(m_lpPrvni)	// pokud je seznam inicializovan
	  {
		  CObouPrvek<T> *lpNovy = new CObouPrvek<T>(_Data);

		  if(lpNovy)
		  {
			  lpNovy->NastavPredchozi(m_lpPrvni);	// novy prvek bude mit jako predchudce zarazku
			  lpNovy->NastavDalsi(m_lpPrvni->VratDalsi());	// a ukazuje na nasledujici
			  m_lpPrvni->VratDalsi()->NastavPredchozi(lpNovy);
			  m_lpPrvni->NastavDalsi(lpNovy);
			  

			  bNavrat = true;
		  }

		  // novy prvek nemazeme
	  }

	  return bNavrat;
  }

  template<class T> bool CObouSeznam<T>::VlozNaKonec(const T& _Data)
  {
	  bool bNavrat = false;

	  if(m_lpPosledni)	// pokud je seznam inicializovan
	  {
		  CObouPrvek<T> *lpNovy = new CObouPrvek<T>(_Data);

		  if(lpNovy)
		  {
			  lpNovy->NastavPredchozi(m_lpPosledni->VratPredchozi());
			  lpNovy->NastavDalsi(m_lpPosledni);
			  m_lpPosledni->VratPredchozi()->NastavDalsi(lpNovy);
			  m_lpPosledni->NastavPredchozi(lpNovy);
			  			  
			  bNavrat = true;
		  }
	  }

	  return bNavrat;
  }

  template<class T> bool CObouSeznam<T>::VlozZa(CObouPrvek<T>* _lpPrvek, const T& _Data)
  {
	  bool bNavrat = false;

	  if(!JePrazdny() && _lpPrvek)
	  {
		  CObouPrvek<T> *lpNovy = new CObouPrvek<T>(_Data);	// vytvorime novy prvek

		  if(lpNovy)
		  {
			  lpNovy->NastavDalsi(_lpPrvek->VratDalsi());	// naslednik prvku z parametru je naslednikem noveho
			  lpNovy->NastavPredchozi(_lpPrvek);	// prvek z parametru je predchudcem noveho prvku
			  _lpPrvek->VratDalsi()->NastavPredchozi(lpNovy);	// novy prvek je predchudce naslednika prvku 
			  _lpPrvek->NastavDalsi(lpNovy);	// novy prvek je za prvkem v parametru
			  
			  bNavrat = true;
		  }

		  // klasicky nesmime prvek smazat
	  }

	  return bNavrat;
  }

  template<class T> bool CObouSeznam<T>::VlozPred(CObouPrvek<T>* _lpPrvek, const T& _Data)
  {
	  bool bNavrat = false;

	  if(!JePrazdny() && _lpPrvek)
	  {
		  CObouPrvek<T> *lpNovy = new CObouPrvek<T>(_Data);

		  if(lpNovy)
		  {
			  lpNovy->NastavDalsi(_lpPrvek);	// puvodni prvek nasleduje za novym
			  lpNovy->NastavPredchozi(_lpPrvek->VratPredchozi());	// puvodni predchudce prvku je predchudcem noveho
			  _lpPrvek->VratPredchozi()->NastavDalsi(lpNovy);	// novy prvek je naslednikem predchudce prvku
			  _lpPrvek->NastavPredchozi(lpNovy);	// predchudcem prvku je novy prvek

			  			  
			  bNavrat = true;
		  }
	  }

	  return bNavrat;
  }

  template<class T> bool CObouSeznam<T>::SmazSeznam()
  {
	  bool bNavrat = false;

	  if(m_lpPrvni && m_lpPosledni)	// pokud je seznam platny, tak ho smazeme ...
	  {
		  CObouPrvek<T> *lpMazany;
		  
		  while(m_lpPrvni)
		  {
			  lpMazany = m_lpPrvni;
			  m_lpPrvni = m_lpPrvni->VratDalsi();	// posun na dalsi prvek
			  delete lpMazany;
		  }

		  m_lpPrvni = NULL;
		  m_lpPosledni = NULL;

		  bNavrat = true;
	  }

	  return bNavrat;
  }

  template<class T> bool CObouSeznam<T>::SmazPrvek(CObouPrvek<T>* _lpPrvek)
  {
	  bool bNavrat = false;

	  if(!JePrazdny() && _lpPrvek)
	  {
		  _lpPrvek->VratPredchozi()->NastavDalsi(_lpPrvek->VratDalsi());
		  _lpPrvek->VratDalsi()->NastavPredchozi(_lpPrvek->VratPredchozi());

		  // Vse v poradku, muzeme prvek smazat
		  delete(_lpPrvek);
		  _lpPrvek = NULL;

		  bNavrat = true;
	  }

	  return bNavrat;
  }

  template<class T> bool CObouSeznam<T>::SmazZa(CObouPrvek<T>* _lpPrvek)
  {
	  bool bNavrat = false;

	  if(!JePrazdny() && _lpPrvek)	// nemuzeme mazat zarazku
	  {
		  CObouPrvek<T> *lpMazany = _lpPrvek->VratDalsi();

		  if(lpMazany && (lpMazany != m_lpPosledni))	// zkontrolujeme jestli prvek existuje a jestli to neni zarazka
		  {
			  bNavrat = SmazPrvek(lpMazany);
		  }
	  }

	  return bNavrat;
  }

  template<class T> bool CObouSeznam<T>::SmazPred(CObouPrvek<T>* _lpPrvek)
  {
	  bool bNavrat = false;

	  if(!JePrazdny() && _lpPrvek)	// nemuzeme mazat zarazku
	  {
		  CObouPrvek<T> *lpMazany = _lpPrvek->VratPredchozi();

		  if(lpMazany && (lpMazany != m_lpPrvni))	// zkontrolujeme jestli prvek existuje a jestli to neni zarazka
		  {
			  bNavrat = SmazPrvek(lpMazany);
		  }
	  }

	  return bNavrat;
  }

  template<class T> void CObouSeznam<T>::VypisOdZacatku()
  {
	  cout << "Stav seznamu: ";

	  if(!JePrazdny())
	  {
		  CObouPrvek<T> *lpVypisovany = m_lpPrvni->VratDalsi();	// prvni zarazka nas nezajima
		  
		  while(lpVypisovany != m_lpPosledni)	// data zarazky pro nas nejsou dulezita
		  {
			  cout << lpVypisovany->VratData() << " ";

			  lpVypisovany = lpVypisovany->VratDalsi();
		  }
	  }
	  else
	  {
		  cout << " PRAZDNY";
	  }

	  cout << endl;
  }

  template<class T> void CObouSeznam<T>::VypisOdKonce()
  {
	  cout << "Stav seznamu: ";

	  if(!JePrazdny())
	  {
		  CObouPrvek<T> *lpVypisovany = m_lpPosledni->VratPredchozi();	// prvni zarazka nas nezajima
		  
		  while(lpVypisovany != m_lpPrvni)	// data zarazky pro nas nejsou dulezita
		  {
			  cout << lpVypisovany->VratData() << " ";

			  lpVypisovany = lpVypisovany->VratPredchozi();
		  }
	  }
	  else
	  {
		  cout << " PRAZDNY";
	  }

	  cout << endl;
  }

  template<class T> CObouPrvek<T>* CObouSeznam<T>::Najdi(const T& _Data)
  {
	  CObouPrvek<T> *lpNavrat = NULL; // implicitne nenalezeno

      if(!JePrazdny())
      {
		  CObouPrvek<T> *lpZkoumany = m_lpPrvni->VratDalsi();	// zarazka nas nezajima

          m_lpPosledni->NastavData(_Data); // ulozime data do zarazky
          while(lpZkoumany->VratData() != _Data)
          {
              lpZkoumany = lpZkoumany->VratDalsi();
          }

          if(lpZkoumany != m_lpPosledni)
          {
              lpNavrat = lpZkoumany; // nalezli jsme prvek
          }
      }

      return lpNavrat;
  }

  template<class T> CObouPrvek<T>* CObouSeznam<T>::NajdiBezZarazky(const T& _Data)
  {
	  CObouPrvek<T> *lpNavrat = NULL;	// implicitne nenalezeno

	  if(!JePrazdny())
	  {
		  CObouPrvek<T> *lpZkoumany = m_lpPrvni->VratDalsi();

		  while(lpZkoumany != m_lpPosledni)
		  {
			  if(_Data == lpZkoumany->VratData())
			  {
				  lpNavrat = lpZkoumany;
				  break;
			  }

			  lpZkoumany = lpZkoumany->VratDalsi();
		  }
	  }
	  
	  return lpNavrat;
  }

  template<class T> CObouPrvek<T>* CObouSeznam<T>::Najdi(const T& _Data, unsigned int _k)
  {
	  CObouPrvek<T> *lpNavrat = NULL;	// implicitne nenalezeno

	  if(!JePrazdny())
	  {
		  CObouPrvek<T> *lpZkoumany = m_lpPrvni->VratDalsi();

		  while((lpZkoumany != m_lpPosledni) && (_k != 0))
		  {
			  if(_Data == lpZkoumany->VratData())
			  {
				  if(--_k == 0)
				  { 
					  lpNavrat = lpZkoumany;
					  break;
				  }
			  }

			  lpZkoumany = lpZkoumany->VratDalsi();
		  }
	  }
	  
	  return lpNavrat;
  }

  template<class T> CObouPrvek<T>* CObouSeznam<T>::NajdiNejmensi(CObouPrvek<T>* _lpPrvek)
  {
	  CObouPrvek<T> *lpNavrat = NULL;	// implicitne nenalezeno

	  if(!JePrazdny())
	  {
		  if(!_lpPrvek)
		  {
			  _lpPrvek = m_lpPrvni->VratDalsi();
		  }

		  CObouPrvek<T> *lpZkoumany = _lpPrvek;

		  // prvni prvek je nejdriv nejmensi
		  T Data = lpZkoumany->VratData();
		  lpNavrat = lpZkoumany;


		  while(lpZkoumany != m_lpPosledni)
		  {
			  if(Data > lpZkoumany->VratData())	// pokud nalezneme mensi
			  {
				  lpNavrat = lpZkoumany;	// tak jsme prozatim nalezli nejmensi prvek
				  Data = lpZkoumany->VratData();	// a mame novou nejmensi hodnotu
			  }

			  lpZkoumany = lpZkoumany->VratDalsi();	// posun na dalsi
		  }
	  }
	  
	  return lpNavrat;
  }

  template<class T> void CObouSeznam<T>::SetridVyberem()
  {
	  if(!JePrazdny())
	  {
		  CObouPrvek<T> *lpPosledni, *lpNejmensi;
		  T Vymena;

		  lpPosledni = m_lpPrvni->VratDalsi();

		  while(lpPosledni->VratDalsi() != m_lpPosledni)
		  {
			  lpNejmensi = NajdiNejmensi(lpPosledni->VratDalsi());

			  if(lpNejmensi && (lpNejmensi->VratData() < lpPosledni->VratData()))
			  {
				  Vymena = lpPosledni->VratData();
				  lpPosledni->NastavData(lpNejmensi->VratData());
				  lpNejmensi->NastavData(Vymena);
			  }

			  lpPosledni = lpPosledni->VratDalsi();
		  }
	  }
  }
  
#endif
